/*
* Copyright (C) 2003-2007 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "SyncResults.h"
#include "HwndFunctions.h"

SyncResults SyncResultsLogger = SyncResults(false);

extern char logmsg[512];

static FILE* logFile = NULL;
static char logFullName[512] = "\\" SYNCRESULTS_FILENAME ;    
static char logName[128] = SYNCRESULTS_FILENAME;
static char logPath[256] = "\\" ;   



/*
* return a the time to write into log file. If complete is true, it return 
* the date too, else only hours, minutes, seconds and milliseconds
*/ 
static char* getCurrentTime(BOOL complete) {
    
    SYSTEMTIME sys_time;   
    TIME_ZONE_INFORMATION timezone;
    GetLocalTime(&sys_time);
    GetTimeZoneInformation(&timezone);

    char fmtComplete[] = "%04d-%02d-%02d %02d:%02d:%02d GMT %c%d:%02d";
    char fmt[]         = "%02d:%02d:%02d GMT %c%d:%02d";

    char*  ret = new char [64];
    
    // calculate offset from UTC/GMT in hours:min, positive value means east of Greenwich (e.g. CET = GMT +1)
    
    char direction = timezone.Bias <= 0 ? '+' : '-';    
    int hours = abs(timezone.Bias / 60) ;
    int minutes = abs(timezone.Bias % 60);

    if (complete) {
        sprintf(ret, fmtComplete, sys_time.wYear, sys_time.wMonth, sys_time.wDay,
                 sys_time.wHour, sys_time.wMinute, sys_time.wSecond,
                direction, hours, minutes);
    } else {
        sprintf(ret, fmt, sys_time.wHour, sys_time.wMinute, sys_time.wSecond, 
                direction, hours, minutes);
    }
    return ret;
}

//---------------

SyncResults::SyncResults(BOOL resetLog, char* path, char* name) 
{
    setLogPath(path);
    setLogName(name);
    if (resetLog) 
    {
        reset("");
    }
    syncTime = getCurrentTime(true);
}

SyncResults::~SyncResults() {
   
    if (logFile != NULL) {
        fclose(logFile);
    }
}

void SyncResults::setLogPath(char* configLogPath) {
    
    if (configLogPath != NULL) {
        sprintf(logPath, "%s/", configLogPath); 
    } else {
        sprintf(logPath, "%s", "./");
    }
}

void SyncResults::setLogName(char* configLogName) {
    
    if (configLogName != NULL) {
        sprintf(logName, "%s", configLogName); 
    }
    else {
        sprintf(logName, "%s", SYNCRESULTS_FILENAME);         
    }
}

void SyncResults::printMessage(const char* msg, ...) 
{           
    logFile = fopen(logFullName, "a+"); 
    
    va_list argList;
    va_start (argList, msg);
    vfprintf(logFile, msg, argList);    
    fprintf(logFile, "\n"); 
    fclose(logFile);
}

void SyncResults::reset(const char* title) 
{
    char* currentTime = (char*) getCurrentTime(true);
    memset(logFullName, 0, 512*sizeof(char));
    sprintf(logFullName, "%s%s", logPath, logName);
    logFile = fopen(logFullName, "w+");      
   
    fprintf(logFile, "Date: %s\n",currentTime);
    fclose(logFile);

    for(int i=0;i<6;i++)
    {
        nrContacts[i] =0; nrCalendars[i]=0; nrTasks[i] =0; nrFiles[i] =0; nrMails[i] =0;
    }
    syncedSources = 0;
    syncTypes = 0;
    currentSource = 0;
    itemNumber = 0;
    totalItems = 0;
    flagTotalItems = 0;
    flagMail = 0;
    delete[] currentTime;    
}


void SyncResults::writeMailSyncResults()
{
  if(syncedSources & SYNCRESULTS_SYNC_MAIL)
  {
    for(int i=0;i<6;i++)
        if (nrMails[i] == -1)
            nrMails[i] = 0;

    SyncResultsLogger.printMessage("Mails");

    SyncResultsLogger.printMessage("  ---- client items ----");
    SyncResultsLogger.printMessage("  new: %d", nrMails[3]);
    SyncResultsLogger.printMessage("  updated: %d", nrMails[4]);
    SyncResultsLogger.printMessage("  deleted: %d", nrMails[5]);
    SyncResultsLogger.printMessage("  ---- server items ----");
    SyncResultsLogger.printMessage("  new: %d", nrMails[0]);
    SyncResultsLogger.printMessage("  updated: %d", nrMails[1]);
    SyncResultsLogger.printMessage("  deleted: %d\n", nrMails[2]);    
  };
}

void SyncResults::writePimSyncResults()
{
    int t=0; // TODO: remove t
    if (syncedSources & SYNCRESULTS_SYNC_CALENDAR)
    {
        SyncResultsLogger.printMessage("Calendar");
        
        SyncResultsLogger.printMessage("  ---- client items ----");
        SyncResultsLogger.printMessage("  new: %i", nrCalendars[3-t]);
        SyncResultsLogger.printMessage("  updated: %i", nrCalendars[4-t]);
        SyncResultsLogger.printMessage("  deleted: %i", nrCalendars[5-t]);
        SyncResultsLogger.printMessage("  ---- server items ----");
        SyncResultsLogger.printMessage("  new: %i", nrCalendars[0+t]);
        SyncResultsLogger.printMessage("  updated: %i", nrCalendars[1+t]);
        SyncResultsLogger.printMessage("  deleted: %i\n", nrCalendars[2+t]);
    }
    
    if (syncedSources & SYNCRESULTS_SYNC_CONTACTS)
    {
        SyncResultsLogger.printMessage("Contacts");
        
        SyncResultsLogger.printMessage("  ---- client items ----");
        SyncResultsLogger.printMessage("  new: %i", nrContacts[3-t]);
        SyncResultsLogger.printMessage("  updated: %i", nrContacts[4-t]);
        SyncResultsLogger.printMessage("  deleted: %i", nrContacts[5-t]);
        SyncResultsLogger.printMessage("  ---- server items ----");
        SyncResultsLogger.printMessage("  new: %i", nrContacts[0+t]);
        SyncResultsLogger.printMessage("  updated: %i", nrContacts[1+t]);
        SyncResultsLogger.printMessage("  deleted: %i\n", nrContacts[2+t]);
    }
        
    if (syncedSources & SYNCRESULTS_SYNC_TASKS)
    {
        SyncResultsLogger.printMessage("Tasks");

        SyncResultsLogger.printMessage("  ---- client items ----");
        SyncResultsLogger.printMessage("  new: %i", nrTasks[3-t]);
        SyncResultsLogger.printMessage("  updated: %i", nrTasks[4-t]);
        SyncResultsLogger.printMessage("  deleted: %i", nrTasks[5-t]);
        SyncResultsLogger.printMessage("  ---- server items ----");
        SyncResultsLogger.printMessage("  new: %i", nrTasks[0+t]);
        SyncResultsLogger.printMessage("  updated: %i", nrTasks[1+t]);
        SyncResultsLogger.printMessage("  deleted: %i\n", nrTasks[2+t]);
    }

    if (syncedSources & SYNCRESULTS_SYNC_FILES)
    {
        SyncResultsLogger.printMessage("Files");
        
        SyncResultsLogger.printMessage("  ---- client items ----");
        SyncResultsLogger.printMessage("  new: %i", nrFiles[3-t]);
        SyncResultsLogger.printMessage("  updated: %i", nrFiles[4-t]);
        SyncResultsLogger.printMessage("  deleted: %i", nrFiles[5-t]);
        SyncResultsLogger.printMessage("  ---- server items ----");
        SyncResultsLogger.printMessage("  new: %i", nrFiles[0+t]);
        SyncResultsLogger.printMessage("  updated: %i", nrFiles[1+t]);
        SyncResultsLogger.printMessage("  deleted: %i\n", nrFiles[2+t]);
    }

    if (syncedSources & SYNCRESULTS_SYNC_NOTES)
    {
        SyncResultsLogger.printMessage("Notes");
        
        SyncResultsLogger.printMessage("  ---- client items ----");
        SyncResultsLogger.printMessage("  new: %i", nrNotes[0]);
        SyncResultsLogger.printMessage("  updated: %i", nrNotes[1]);
        SyncResultsLogger.printMessage("  deleted: %i", nrNotes[2]);
        SyncResultsLogger.printMessage("  ---- server items ----");
        SyncResultsLogger.printMessage("  new: %i", nrNotes[3]);
        SyncResultsLogger.printMessage("  updated: %i", nrNotes[4]);
        SyncResultsLogger.printMessage("  deleted: %i\n", nrNotes[5]);
    }

}

void SyncResults::updatePim(const char* item, int values[6],bool fromClient)
{
    if(strcmp(item,"contacts") == 0)
    {
        for(int i=0;i<6;i++)
            nrContacts[i] += values[i];
        syncedSources |= SYNCRESULTS_SYNC_CONTACTS;
    }

    if(strcmp(item,"calendars") == 0)
    {
        for(int i=0;i<6;i++)
            nrCalendars[i] += values[i];
        syncedSources |= SYNCRESULTS_SYNC_CALENDAR;
    }

    if(strcmp(item,"tasks") == 0)
    {
        for(int i=0;i<6;i++)
            nrTasks[i] += values[i];
        syncedSources |= SYNCRESULTS_SYNC_TASKS;
    }

    if(strcmp(item,"files") == 0)
    {
        for(int i=0;i<6;i++)
            nrFiles[i] += values[i];
        syncedSources |= SYNCRESULTS_SYNC_FILES;
    }

    if(strcmp(item,"notes") == 0)
    {
        for(int i=0;i<6;i++)
            nrNotes[i] += values[i];
        syncedSources |= SYNCRESULTS_SYNC_NOTES;
    }


    totalItems = 0;
    for (int i=0;i<3;i++){
            totalItems += values[i];
        }

    if(fromClient){
            if(HwndFunctions::wnd)
                SendMessage(HwndFunctions::wnd, (WM_APP+8), (WPARAM) -1,(LPARAM)totalItems);
    }
}

void SyncResults::updateMails(int values[6])
{
    for(int i=0;i<6;i++)
        nrMails[i] += values[i];

    totalItems = 0;
    for (int i=0;i<3;i++)
    {
         totalItems+= nrMails[i];
    }

    if(flagTotalItems == 2)
        //-- send the total number of items to sync
        if(HwndFunctions::wnd)
            SendMessage(HwndFunctions::wnd, (WM_APP+8), -1,(LPARAM)totalItems);
    
    flagTotalItems++;
    if(flagTotalItems == 3)
        flagTotalItems = 0; 

    syncedSources |= SYNCRESULTS_SYNC_MAIL;
}

void SyncResults::incMails(int index)
{
    nrMails[index]++;
    syncedSources |= SYNCRESULTS_SYNC_MAIL;
}